@@ -21,11 +21,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#define _GNU_SOURCE
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdio.h>
 
 #include "conf.h"
 #include "log.h"
@@ -41,22 +43,29 @@ struct execute_args {
 
 static int execute_start(struct lxc_handler *handler, void* data)
 {
-	int j, i = 0;
-	struct execute_args *my_args = data;
+	int argc_add, j;
 	char **argv;
-	int argc = 0, argc_add;
+	int argc = 0, i = 0, logfd = -1;
+	struct execute_args *my_args = data;
 	char *initpath;
+	char logfile[LXC_PROC_PID_FD_LEN];
 
-	while (my_args->argv[argc++]);
+	while (my_args->argv[argc++])
+		;
 
-	argc_add = 4;
+	/* lxc-init -n name -- [argc] NULL -> 5 */
+	argc_add = 5;
 	if (my_args->quiet)
 		argc_add++;
-	if (!handler->conf->rootfs.path) {
-		argc_add += 4;
-		if (lxc_log_has_valid_level())
-			argc_add += 2;
-	}
+
+	if (!handler->conf->rootfs.path)
+		argc_add += 2;
+
+	if (lxc_log_has_valid_level())
+		argc_add += 2;
+
+	if (current_config->logfd != -1 || lxc_log_fd != -1)
+		argc_add += 2;
 
 	argv = malloc((argc + argc_add) * sizeof(*argv));
 	if (!argv)
@@ -64,34 +73,62 @@ static int execute_start(struct lxc_handler *handler, void* data)
 
 	initpath = choose_init(NULL);
 	if (!initpath) {
-		ERROR("Failed to find an lxc-init or init.lxc");
+		ERROR("Failed to find an init.lxc or init.lxc.static");
 		goto out2;
 	}
 	argv[i++] = initpath;
+
+	argv[i++] = "-n";
+	argv[i++] = (char *)handler->name;
+
+	if (lxc_log_has_valid_level()) {
+		argv[i++] = "-l";
+		argv[i++] =
+		    (char *)lxc_log_priority_to_string(lxc_log_get_level());
+	}
+
+	if (current_config->logfd != -1 || lxc_log_fd != -1) {
+		int ret;
+		int to_dup = current_config->logfd;
+
+		if (current_config->logfd == -1)
+			to_dup = lxc_log_fd;
+
+		logfd = dup(to_dup);
+		if (logfd < 0) {
+			SYSERROR("Failed to duplicate log file descriptor");
+			goto out2;
+		}
+
+		ret = snprintf(logfile, sizeof(logfile), "/proc/1/fd/%d", logfd);
+		if (ret < 0 || (size_t)ret >= sizeof(logfile))
+			goto out3;
+
+		argv[i++] = "-o";
+		argv[i++] = logfile;
+	}
+
 	if (my_args->quiet)
 		argv[i++] = "--quiet";
-	if (!handler->conf->rootfs.path) {
-		argv[i++] = "--name";
-		argv[i++] = (char *)handler->name;
-		argv[i++] = "--lxcpath";
-		argv[i++] = (char *)handler->lxcpath;
 
-		if (lxc_log_has_valid_level()) {
-			argv[i++] = "--logpriority";
-			argv[i++] = (char *)
-				lxc_log_priority_to_string(lxc_log_get_level());
-		}
+	if (!handler->conf->rootfs.path) {
+		argv[i++] = "-P";
+		argv[i++] = (char *)handler->lxcpath;
 	}
+
 	argv[i++] = "--";
 	for (j = 0; j < argc; j++)
 		argv[i++] = my_args->argv[j];
 	argv[i++] = NULL;
 
-	NOTICE("exec'ing '%s'", my_args->argv[0]);
+	NOTICE("Exec'ing \"%s\"", my_args->argv[0]);
 
 	execvp(argv[0], argv);
-	SYSERROR("failed to exec %s", argv[0]);
+	SYSERROR("Failed to exec %s", argv[0]);
 	free(initpath);
+
+out3:
+	close(logfd);
 out2:
 	free(argv);
 out1:
@@ -111,16 +148,12 @@ static struct lxc_operations execute_start_ops = {
 };
 
 int lxc_execute(const char *name, char *const argv[], int quiet,
-		struct lxc_conf *conf, const char *lxcpath, bool backgrounded)
+		struct lxc_handler *handler, const char *lxcpath,
+		bool backgrounded, int *error_num)
 {
-	struct execute_args args = {
-		.argv = argv,
-		.quiet = quiet
-	};
+	struct execute_args args = {.argv = argv, .quiet = quiet};
 
-	if (lxc_check_inherited(conf, false, -1))
-		return -1;
-
-	conf->is_execute = 1;
-	return __lxc_start(name, conf, &execute_start_ops, &args, lxcpath, backgrounded);
+	handler->conf->is_execute = 1;
+	return __lxc_start(name, handler, &execute_start_ops, &args, lxcpath,
+			   backgrounded, error_num);
 }
