@@ -273,6 +273,49 @@ ATTR_UNUSED static inline void LXC_##LEVEL(struct lxc_log_locinfo* locinfo,	\
 #define lxc_log_category_priority(name) 				\
 	(lxc_log_priority_to_string(lxc_log_category_##name.priority))
 
+/*
+ * Helper macro to define errno string.
+ */
+#if HAVE_STRERROR_R
+	#ifndef HAVE_DECL_STRERROR_R
+		#ifdef STRERROR_R_CHAR_P
+			char *strerror_r(int errnum, char *buf, size_t buflen);
+		#else
+			int strerror_r(int errnum, char *buf, size_t buflen);
+		#endif
+	#endif
+
+	#ifdef STRERROR_R_CHAR_P
+		#define lxc_log_strerror_r                                               \
+			char errno_buf[PATH_MAX / 2] = {"Failed to get errno string"};   \
+			char *ptr = NULL;                                                \
+			{                                                                \
+				int saved_errno = errno;				 \
+				ptr = strerror_r(errno, errno_buf, sizeof(errno_buf));   \
+				errno = saved_errno;					 \
+				if (!ptr)                                                \
+					ptr = errno_buf;                                 \
+			}
+	#else
+		#define lxc_log_strerror_r                                               \
+			char errno_buf[PATH_MAX / 2] = {"Failed to get errno string"};   \
+			char *ptr = errno_buf;                                           \
+			{                                                                \
+				int saved_errno = errno;				 \
+				(void)strerror_r(errno, errno_buf, sizeof(errno_buf));   \
+				errno = saved_errno;					 \
+			}
+	#endif
+#elif ENFORCE_THREAD_SAFETY
+	#error ENFORCE_THREAD_SAFETY was set but cannot be guaranteed
+#else
+	#define lxc_log_strerror_r							 \
+		char *ptr = NULL;              						 \
+		{                              						 \
+			ptr = strerror(errno); 						 \
+		}
+#endif
+
 /*
  * top categories
  */
@@ -321,11 +364,41 @@ ATTR_UNUSED static inline void LXC_##LEVEL(struct lxc_log_locinfo* locinfo,	\
 	LXC_FATAL(&locinfo, format, ##__VA_ARGS__);			\
 } while (0)
 
+#define SYSTRACE(format, ...)                              \
+	do {                                               \
+		lxc_log_strerror_r;                        \
+		TRACE("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
 
+#define SYSDEBUG(format, ...)                              \
+	do {                                               \
+		lxc_log_strerror_r;                        \
+		DEBUG("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
 
-#define SYSERROR(format, ...) do {				    	\
-	ERROR("%s - " format, strerror(errno), ##__VA_ARGS__);		\
-} while (0)
+#define SYSINFO(format, ...)                              \
+	do {                                              \
+		lxc_log_strerror_r;                       \
+		INFO("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
+
+#define SYSNOTICE(format, ...)                              \
+	do {                                                \
+		lxc_log_strerror_r;                         \
+		NOTICE("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
+
+#define SYSWARN(format, ...)                              \
+	do {                                              \
+		lxc_log_strerror_r;                       \
+		WARN("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
+
+#define SYSERROR(format, ...)                              \
+	do {                                               \
+		lxc_log_strerror_r;                        \
+		ERROR("%s - " format, ptr, ##__VA_ARGS__); \
+	} while (0)
 
 extern int lxc_log_fd;
 
