@@ -59,6 +59,10 @@
 #include <../include/ifaddrs.h>
 #endif
 
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
 #ifndef IFLA_LINKMODE
 #define IFLA_LINKMODE 17
 #endif
@@ -119,7 +123,10 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 		memcpy(netdev->priv.veth_attr.veth1, veth1, IFNAMSIZ);
 	}
 
-	snprintf(veth2buf, sizeof(veth2buf), "vethXXXXXX");
+	err = snprintf(veth2buf, sizeof(veth2buf), "vethXXXXXX");
+	if (err < 0 || (size_t)err >= sizeof(veth2buf))
+		return -1;
+
 	veth2 = lxc_mkifname(veth2buf);
 	if (!veth2)
 		goto out_delete;
@@ -533,7 +540,8 @@ static char * is_wlan(const char *ifname)
 	fseek(f, 0, SEEK_END);
 	physlen = ftell(f);
 	fseek(f, 0, SEEK_SET);
-	physname = malloc(physlen+1);
+
+	physname = malloc(physlen + 1);
 	if (!physname) {
 		fclose(f);
 		goto bad;
@@ -1291,43 +1299,17 @@ static int proc_sys_net_write(const char *path, const char *value)
 	if (fd < 0)
 		return -errno;
 
-	if (write(fd, value, strlen(value)) < 0)
+	if (lxc_write_nointr(fd, value, strlen(value)) < 0)
 		err = -errno;
 
 	close(fd);
 	return err;
 }
 
-static int ip_forward_set(const char *ifname, int family, int flag)
-{
-	char path[MAXPATHLEN];
-	int rc;
-
-	if (family != AF_INET && family != AF_INET6)
-		return -EINVAL;
-
-	rc = snprintf(path, MAXPATHLEN, "/proc/sys/net/%s/conf/%s/forwarding",
-		 family == AF_INET?"ipv4":"ipv6" , ifname);
-	if (rc >= MAXPATHLEN)
-		return -E2BIG;
-
-	return proc_sys_net_write(path, flag?"1":"0");
-}
-
-int lxc_ip_forward_on(const char *ifname, int family)
-{
-	return ip_forward_set(ifname, family, 1);
-}
-
-int lxc_ip_forward_off(const char *ifname, int family)
-{
-	return ip_forward_set(ifname, family, 0);
-}
-
 static int neigh_proxy_set(const char *ifname, int family, int flag)
 {
-	char path[MAXPATHLEN];
 	int ret;
+	char path[MAXPATHLEN];
 
 	if (family != AF_INET && family != AF_INET6)
 		return -EINVAL;
@@ -1841,7 +1823,8 @@ static int lxc_ovs_attach_bridge(const char *bridge, const char *nic)
 
 int lxc_bridge_attach(const char *bridge, const char *ifname)
 {
-	int fd, index, err;
+	int err, fd, index;
+	size_t retlen;
 	struct ifreq ifr;
 
 	if (strlen(ifname) >= IFNAMSIZ)
@@ -1858,8 +1841,12 @@ int lxc_bridge_attach(const char *bridge, const char *ifname)
 	if (fd < 0)
 		return -errno;
 
-	strncpy(ifr.ifr_name, bridge, IFNAMSIZ-1);
-	ifr.ifr_name[IFNAMSIZ-1] = '\0';
+	retlen = strlcpy(ifr.ifr_name, bridge, IFNAMSIZ);
+	if (retlen >= IFNAMSIZ) {
+		close(fd);
+		return -E2BIG;
+	}
+
 	ifr.ifr_ifindex = index;
 	err = ioctl(fd, SIOCBRADDIF, &ifr);
 	close(fd);
@@ -1889,6 +1876,7 @@ static const char padchar[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 char *lxc_mkifname(char *template)
 {
+	int ret;
 	unsigned int seed;
 	FILE *urandom;
 	struct ifaddrs *ifa, *ifaddr;
@@ -1900,7 +1888,11 @@ char *lxc_mkifname(char *template)
 		return NULL;
 
 	/* Get all the network interfaces. */
-	getifaddrs(&ifaddr);
+	ret = getifaddrs(&ifaddr);
+	if (ret < 0) {
+		ERROR("%s - Failed to get network interfaces", strerror(errno));
+		return NULL;
+	}
 
 	/* Initialize the random number generator. */
 	urandom = fopen("/dev/urandom", "r");
@@ -1919,7 +1911,7 @@ char *lxc_mkifname(char *template)
 	/* Generate random names until we find one that doesn't exist. */
 	while (true) {
 		name[0] = '\0';
-		strcpy(name, template);
+		(void)strlcpy(name, template, IFNAMSIZ);
 
 		exists = false;
 		for (i = 0; i < strlen(name); i++) {
@@ -1944,7 +1936,9 @@ char *lxc_mkifname(char *template)
 	}
 
 	freeifaddrs(ifaddr);
-	return strcpy(template, name);
+	(void)strlcpy(template, name, strlen(template) + 1);
+
+	return template;
 }
 
 int setup_private_host_hw_addr(char *veth1)
@@ -1957,7 +1951,12 @@ int setup_private_host_hw_addr(char *veth1)
 	if (sockfd < 0)
 		return -errno;
 
-	snprintf((char *)ifr.ifr_name, IFNAMSIZ, "%s", veth1);
+	err = snprintf((char *)ifr.ifr_name, IFNAMSIZ, "%s", veth1);
+	if (err < 0 || (size_t)err >= IFNAMSIZ) {
+		close(sockfd);
+		return -E2BIG;
+	}
+
 	err = ioctl(sockfd, SIOCGIFHWADDR, &ifr);
 	if (err < 0) {
 		close(sockfd);
@@ -2022,15 +2021,16 @@ int lxc_find_gateway_addresses(struct lxc_handler *handler)
 }
 
 #define LXC_USERNIC_PATH LIBEXECDIR "/lxc/lxc-user-nic"
-static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
+static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcname,
 					  struct lxc_netdev *netdev, pid_t pid)
 {
 	int ret;
 	pid_t child;
 	int bytes, pipefd[2];
 	char *token, *saveptr = NULL;
-	char netdev_link[IFNAMSIZ + 1];
+	char netdev_link[IFNAMSIZ];
 	char buffer[MAXPATHLEN] = {0};
+	size_t retlen;
 
 	if (netdev->type != LXC_NET_VETH) {
 		ERROR("Network type %d not support for unprivileged use", netdev->type);
@@ -2053,6 +2053,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 
 	if (child == 0) {
 		int ret;
+		size_t retlen;
 		char pidstr[LXC_NUMSTRLEN64];
 
 		close(pipefd[0]);
@@ -2063,17 +2064,21 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		close(pipefd[1]);
 		if (ret < 0) {
 			SYSERROR("Failed to duplicate std{err,out} file descriptor");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->link[0] != '\0')
-			strncpy(netdev_link, netdev->link, IFNAMSIZ);
+			retlen = strlcpy(netdev_link, netdev->link, IFNAMSIZ);
 		else
-			strncpy(netdev_link, "none", IFNAMSIZ);
+			retlen = strlcpy(netdev_link, "none", IFNAMSIZ);
+		if (retlen >= IFNAMSIZ) {
+			SYSERROR("Invalid network device name");
+			_exit(EXIT_FAILURE);
+		}
 
 		ret = snprintf(pidstr, LXC_NUMSTRLEN64, "%d", pid);
 		if (ret < 0 || ret >= LXC_NUMSTRLEN64)
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		pidstr[LXC_NUMSTRLEN64 - 1] = '\0';
 
 		INFO("Execing lxc-user-nic create %s %s %s veth %s %s", lxcpath,
@@ -2088,23 +2093,23 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 			       lxcpath, lxcname, pidstr, "veth", netdev_link,
 			       (char *)NULL);
 		SYSERROR("Failed to execute lxc-user-nic");
-		exit(EXIT_FAILURE);
+		_exit(EXIT_FAILURE);
 	}
 
 	/* close the write-end of the pipe */
 	close(pipefd[1]);
 
-	bytes = read(pipefd[0], &buffer, MAXPATHLEN);
+	bytes = lxc_read_nointr(pipefd[0], &buffer, MAXPATHLEN);
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor");
 		close(pipefd[0]);
-		return -1;
+	} else {
+		buffer[bytes - 1] = '\0';
 	}
-	buffer[bytes - 1] = '\0';
 
 	ret = wait_for_pid(child);
 	close(pipefd[0]);
-	if (ret != 0) {
+	if (ret != 0 || bytes < 0) {
 		ERROR("lxc-user-nic failed to configure requested network: %s",
 		      buffer[0] != '\0' ? buffer : "(null)");
 		return -1;
@@ -2118,8 +2123,8 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		return -1;
 	}
 
-	memset(netdev->name, 0, IFNAMSIZ + 1);
-	strncpy(netdev->name, token, IFNAMSIZ);
+	memset(netdev->name, 0, IFNAMSIZ);
+	memcpy(netdev->name, token, IFNAMSIZ - 1);
 
 	/* netdev->ifindex */
 	token = strtok_r(NULL, ":", &saveptr);
@@ -2142,12 +2147,12 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		return -1;
 	}
 
-	if (strlen(token) >= IFNAMSIZ) {
+	retlen = strlcpy(netdev->priv.veth_attr.veth1, token, IFNAMSIZ);
+	if (retlen >= IFNAMSIZ) {
 		ERROR("Host side veth device name returned by lxc-user-nic is "
 		      "too long");
 		return -E2BIG;
 	}
-	strcpy(netdev->priv.veth_attr.veth1, token);
 
 	/* netdev->priv.veth_attr.ifindex */
 	token = strtok_r(NULL, ":", &saveptr);
@@ -2166,7 +2171,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 	return 0;
 }
 
-static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
+static int lxc_delete_network_unpriv_exec(const char *lxcpath, const char *lxcname,
 					  struct lxc_netdev *netdev,
 					  const char *netns_path)
 {
@@ -2206,7 +2211,7 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		close(pipefd[1]);
 		if (ret < 0) {
 			SYSERROR("Failed to duplicate std{err,out} file descriptor");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->priv.veth_attr.pair[0] != '\0')
@@ -2215,13 +2220,13 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 			hostveth = netdev->priv.veth_attr.veth1;
 		if (hostveth[0] == '\0') {
 			SYSERROR("Host side veth device name is missing");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->link[0] == '\0') {
 			SYSERROR("Network link for network device \"%s\" is "
 				 "missing", netdev->priv.veth_attr.veth1);
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		INFO("Execing lxc-user-nic delete %s %s %s veth %s %s", lxcpath,
@@ -2230,28 +2235,27 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		       lxcname, netns_path, "veth", netdev->link, hostveth,
 		       (char *)NULL);
 		SYSERROR("Failed to exec lxc-user-nic.");
-		exit(EXIT_FAILURE);
+		_exit(EXIT_FAILURE);
 	}
 
 	close(pipefd[1]);
 
-	bytes = read(pipefd[0], &buffer, MAXPATHLEN);
+	bytes = lxc_read_nointr(pipefd[0], &buffer, MAXPATHLEN);
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor.");
 		close(pipefd[0]);
-		return -1;
+	} else {
+		buffer[bytes - 1] = '\0';
 	}
-	buffer[bytes - 1] = '\0';
 
-	if (wait_for_pid(child) != 0) {
+	ret = wait_for_pid(child);
+	close(pipefd[0]);
+	if (ret != 0 || bytes < 0) {
 		ERROR("lxc-user-nic failed to delete requested network: %s",
 		      buffer[0] != '\0' ? buffer : "(null)");
-		close(pipefd[0]);
 		return -1;
 	}
 
-	close(pipefd[0]);
-
 	return 0;
 }
 
@@ -2281,7 +2285,7 @@ bool lxc_delete_network_unpriv(struct lxc_handler *handler)
 	}
 
 	ret = snprintf(netns_path, sizeof(netns_path), "/proc/%d/fd/%d",
-		       getpid(), handler->netnsfd);
+		       lxc_raw_getpid(), handler->netnsfd);
 	if (ret < 0 || ret >= sizeof(netns_path))
 		return false;
 
@@ -2380,14 +2384,14 @@ int lxc_create_network_priv(struct lxc_handler *handler)
 	return 0;
 }
 
-int lxc_network_move_created_netdev_priv(const char *lxcpath, char *lxcname,
+int lxc_network_move_created_netdev_priv(const char *lxcpath, const char *lxcname,
 					 struct lxc_list *network, pid_t pid)
 {
 	int ret;
 	char ifname[IFNAMSIZ];
 	struct lxc_list *iterator;
 
-	if (am_unpriv())
+	if (am_guest_unpriv())
 		return 0;
 
 	lxc_list_for_each(iterator, network) {
@@ -2420,12 +2424,12 @@ int lxc_network_move_created_netdev_priv(const char *lxcpath, char *lxcname,
 	return 0;
 }
 
-int lxc_create_network_unpriv(const char *lxcpath, char *lxcname,
+int lxc_create_network_unpriv(const char *lxcpath, const char *lxcname,
 			      struct lxc_list *network, pid_t pid)
 {
 	struct lxc_list *iterator;
 
-	if (!am_unpriv())
+	if (!am_guest_unpriv())
 		return 0;
 
 	lxc_list_for_each(iterator, network) {
@@ -2605,7 +2609,7 @@ int lxc_restore_phys_nics_to_netns(struct lxc_handler *handler)
 
 	TRACE("Moving physical network devices back to parent network namespace");
 
-	oldfd = lxc_preserve_ns(getpid(), "net");
+	oldfd = lxc_preserve_ns(lxc_raw_getpid(), "net");
 	if (oldfd < 0) {
 		SYSERROR("Failed to preserve network namespace");
 		return -1;
@@ -2784,9 +2788,9 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 	 */
 	if (netdev->name[0] == '\0') {
 		if (netdev->type == LXC_NET_PHYS)
-			strcpy(netdev->name, netdev->link);
+			(void)strlcpy(netdev->name, netdev->link, IFNAMSIZ);
 		else
-			strcpy(netdev->name, "eth%d");
+			(void)strlcpy(netdev->name, "eth%d", IFNAMSIZ);
 	}
 
 	/* rename the interface name */
@@ -2812,7 +2816,7 @@ static int lxc_setup_netdev_in_child_namespaces(struct lxc_netdev *netdev)
 	 * name of the network device in the child's network namespace. We will
 	 * later on send this information back to the parent.
 	 */
-	strcpy(netdev->name, current_ifname);
+	(void)strlcpy(netdev->name, current_ifname, IFNAMSIZ);
 
 	/* set a mac address */
 	if (netdev->hwaddr) {
