@@ -41,6 +41,14 @@
 #include "storage.h"
 #include "utils.h"
 
+#ifndef HAVE_STRLCAT
+#include "include/strlcat.h"
+#endif
+
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
 lxc_log_define(btrfs, lxc);
 
 /* defined in lxccontainer.c: needs to become common helper */
@@ -54,11 +62,11 @@ extern char *dir_new_path(char *src, const char *oldname, const char *name,
  * simply return a.
  */
 char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
-			    int name_len)
+			    u16 name_len)
 {
 	struct btrfs_ioctl_ino_lookup_args args;
-	int ret, e;
-	size_t len;
+	int ret;
+	size_t len, retlen;
 	char *retpath;
 
 	memset(&args, 0, sizeof(args));
@@ -66,17 +74,16 @@ char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
 	args.objectid = objid;
 
 	ret = ioctl(fd, BTRFS_IOC_INO_LOOKUP, &args);
-	e = errno;
 	if (ret) {
-		ERROR("Failed to lookup path for %llu %llu %s - %s\n",
-				 (unsigned long long) dir_id,
-				 (unsigned long long) objid,
-				 name, strerror(e));
+		SYSERROR("Failed to lookup path for %llu %llu %s",
+		         (unsigned long long) dir_id,
+		         (unsigned long long) objid,
+		         name);
 		return NULL;
 	} else
-		INFO("Got path for %llu %llu - %s\n",
-			(unsigned long long) objid, (unsigned long long) dir_id,
-			name);
+		INFO("Got path for %llu %llu - %s",
+		     (unsigned long long) objid, (unsigned long long) dir_id,
+		     name);
 
 	if (args.name[0]) {
 		/*
@@ -87,18 +94,33 @@ char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
 		retpath = malloc(len);
 		if (!retpath)
 			return NULL;
-		strcpy(retpath, args.name);
-		strcat(retpath, "/");
-		strncat(retpath, name, name_len);
+
+		(void)strlcpy(retpath, args.name, len);
+		(void)strlcat(retpath, "/", len);
+
+		retlen = strlcat(retpath, name, len);
+		if (retlen >= len) {
+			ERROR("Failed to append name - %s", name);
+			free(retpath);
+			return NULL;
+		}
 	} else {
 		/* we're at the root of ref_tree */
 		len = name_len + 1;
 		retpath = malloc(len);
 		if (!retpath)
 			return NULL;
+
 		*retpath = '\0';
-		strncat(retpath, name, name_len);
+
+		retlen = strlcat(retpath, name, len);
+		if (retlen >= len) {
+			ERROR("Failed to append name - %s", name);
+			free(retpath);
+			return NULL;
+		}
 	}
+
 	return retpath;
 }
 
@@ -220,38 +242,46 @@ int btrfs_umount(struct lxc_storage *bdev)
 
 static int btrfs_subvolume_create(const char *path)
 {
-	int ret, fd = -1;
-	struct btrfs_ioctl_vol_args  args;
-	char *p, *newfull = strdup(path);
+	int ret, saved_errno;
+	size_t retlen;
+	struct btrfs_ioctl_vol_args args;
+	char *p, *newfull;
+	int fd = -1;
 
+	newfull = strdup(path);
 	if (!newfull) {
-		ERROR("Error: out of memory");
-		return -1;
+		errno = ENOMEM;
+		return -ENOMEM;
 	}
 
 	p = strrchr(newfull, '/');
 	if (!p) {
-		ERROR("bad path: %s", path);
 		free(newfull);
-		return -1;
+		errno = EINVAL;
+		return -EINVAL;
 	}
 	*p = '\0';
 
 	fd = open(newfull, O_RDONLY);
 	if (fd < 0) {
-		ERROR("Error opening %s", newfull);
 		free(newfull);
-		return -1;
+		return -errno;
 	}
 
 	memset(&args, 0, sizeof(args));
-	strncpy(args.name, p+1, BTRFS_SUBVOL_NAME_MAX);
-	args.name[BTRFS_SUBVOL_NAME_MAX-1] = 0;
+	retlen = strlcpy(args.name, p + 1, BTRFS_SUBVOL_NAME_MAX);
+	if (retlen >= BTRFS_SUBVOL_NAME_MAX) {
+		free(newfull);
+		close(fd);
+		return -E2BIG;
+	}
+
 	ret = ioctl(fd, BTRFS_IOC_SUBVOL_CREATE, &args);
-	INFO("btrfs: snapshot create ioctl returned %d", ret);
+	saved_errno = errno;
 
-	free(newfull);
 	close(fd);
+	free(newfull);
+	errno = saved_errno;
 	return ret;
 }
 
@@ -298,46 +328,52 @@ out:
 
 int btrfs_snapshot(const char *orig, const char *new)
 {
+	size_t retlen;
+	struct btrfs_ioctl_vol_args_v2 args;
+	char *newdir, *newname;
+	char *newfull = NULL;
+	int saved_errno = -1;
 	int fd = -1, fddst = -1, ret = -1;
-	struct btrfs_ioctl_vol_args_v2  args;
-	char *newdir, *newname, *newfull = NULL;
 
 	newfull = strdup(new);
-	if (!newfull) {
-		ERROR("Error: out of memory");
+	if (!newfull)
 		goto out;
-	}
-	// make sure the directory doesn't already exist
-	if (rmdir(newfull) < 0 && errno != ENOENT) {
-		SYSERROR("Error removing empty new rootfs");
+
+	ret = rmdir(newfull);
+	if (ret < 0 && errno != ENOENT)
 		goto out;
-	}
+
 	newname = basename(newfull);
-	newdir = dirname(newfull);
 	fd = open(orig, O_RDONLY);
-	if (fd < 0) {
-		SYSERROR("Error opening original rootfs %s", orig);
+	if (fd < 0)
 		goto out;
-	}
+
+	newdir = dirname(newfull);
 	fddst = open(newdir, O_RDONLY);
-	if (fddst < 0) {
-		SYSERROR("Error opening new container dir %s", newdir);
+	if (fddst < 0)
 		goto out;
-	}
 
 	memset(&args, 0, sizeof(args));
 	args.fd = fd;
-	strncpy(args.name, newname, BTRFS_SUBVOL_NAME_MAX);
-	args.name[BTRFS_SUBVOL_NAME_MAX-1] = 0;
+	retlen = strlcpy(args.name, newname, BTRFS_SUBVOL_NAME_MAX);
+	if (retlen >= BTRFS_SUBVOL_NAME_MAX)
+		goto out;
+
 	ret = ioctl(fddst, BTRFS_IOC_SNAP_CREATE_V2, &args);
-	INFO("btrfs: snapshot create ioctl returned %d", ret);
+	saved_errno = errno;
 
 out:
 	if (fddst != -1)
 		close(fddst);
+
 	if (fd != -1)
 		close(fd);
+
 	free(newfull);
+
+	if (saved_errno >= 0)
+		errno = saved_errno;
+
 	return ret;
 }
 
@@ -393,7 +429,7 @@ int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
 
 	if (snap) {
 		struct rsync_data_char sdata;
-		if (!am_unpriv())
+		if (!am_guest_unpriv())
 			return btrfs_snapshot(orig->dest, new->dest);
 		sdata.dest = new->dest;
 		sdata.src = orig->dest;
@@ -412,6 +448,7 @@ int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
 static int btrfs_do_destroy_subvol(const char *path)
 {
 	int ret, fd = -1;
+	size_t retlen;
 	struct btrfs_ioctl_vol_args  args;
 	char *p, *newfull = strdup(path);
 
@@ -436,8 +473,13 @@ static int btrfs_do_destroy_subvol(const char *path)
 	}
 
 	memset(&args, 0, sizeof(args));
-	strncpy(args.name, p+1, BTRFS_SUBVOL_NAME_MAX);
-	args.name[BTRFS_SUBVOL_NAME_MAX-1] = 0;
+	retlen = strlcpy(args.name, p+1, BTRFS_SUBVOL_NAME_MAX);
+	if (retlen >= BTRFS_SUBVOL_NAME_MAX) {
+		free(newfull);
+		close(fd);
+		return -E2BIG;
+	}
+
 	ret = ioctl(fd, BTRFS_IOC_SNAP_DESTROY, &args);
 	INFO("btrfs: snapshot destroy ioctl returned %d for %s", ret, path);
 	if (ret < 0 && errno == EPERM)
@@ -491,22 +533,29 @@ static bool update_tree_node(struct mytree_node *n, u64 id, u64 parent,
 {
 	if (id)
 		n->objid = id;
+
 	if (parent)
 		n->parentid = parent;
+
 	if (name) {
 		n->name = malloc(name_len + 1);
 		if (!n->name)
 			return false;
-		strncpy(n->name, name, name_len);
-		n->name[name_len] = '\0';
+
+		(void)strlcpy(n->name, name, name_len + 1);
 	}
+
 	if (dirname) {
-		n->dirname = malloc(strlen(dirname) + 1);
+		size_t len;
+
+		len = strlen(dirname);
+		n->dirname = malloc(len + 1);
 		if (!n->dirname) {
 			free(n->name);
 			return false;
 		}
-		strcpy(n->dirname, dirname);
+
+		(void)strlcpy(n->dirname, dirname, len + 1);
 	}
 	return true;
 }
@@ -553,7 +602,7 @@ static void free_btrfs_tree(struct my_btrfs_tree *tree)
 static bool do_remove_btrfs_children(struct my_btrfs_tree *tree, u64 root_id,
 				     const char *path)
 {
-	int i;
+	int i, ret;
 	char *newpath;
 	size_t len;
 
@@ -569,7 +618,11 @@ static bool do_remove_btrfs_children(struct my_btrfs_tree *tree, u64 root_id,
 				ERROR("Out of memory");
 				return false;
 			}
-			snprintf(newpath, len, "%s/%s", path, tree->nodes[i].dirname);
+			ret = snprintf(newpath, len, "%s/%s", path, tree->nodes[i].dirname);
+			if (ret < 0 || ret >= len) {
+				free(newpath);
+				return false;
+			}
 			if (!do_remove_btrfs_children(tree, tree->nodes[i].objid, newpath)) {
 				ERROR("Failed to prune %s\n", tree->nodes[i].name);
 				free(newpath);
