@@ -61,6 +61,10 @@
 #include "utils.h"
 #include "zfs.h"
 
+#ifndef HAVE_STRLCPY
+#include "include/strlcpy.h"
+#endif
+
 #ifndef BLKGETSIZE64
 #define BLKGETSIZE64 _IOR(0x12, 114, size_t)
 #endif
@@ -372,7 +376,7 @@ struct lxc_storage *storage_copy(struct lxc_container *c0, const char *cname,
 	if (!bdevtype && !keepbdevtype && snap && (!strcmp(orig->type, "dir") || !strcmp(orig->type, "overlayfs")))
 		bdevtype = "overlayfs";
 
-	if (am_unpriv() && !unpriv_snap_allowed(orig, bdevtype, snap, maybe_snap)) {
+	if (am_guest_unpriv() && !unpriv_snap_allowed(orig, bdevtype, snap, maybe_snap)) {
 		ERROR("Unsupported snapshot type \"%s\" for unprivileged users",
 		      bdevtype ? bdevtype : "(null)");
 		storage_put(orig);
@@ -409,7 +413,7 @@ struct lxc_storage *storage_copy(struct lxc_container *c0, const char *cname,
 		goto err;
 	}
 
-	if (am_unpriv() && chown_mapped_root(new->src, c0->lxc_conf) < 0)
+	if (am_guest_unpriv() && chown_mapped_root(new->src, c0->lxc_conf) < 0)
 		WARN("Failed to update ownership of %s", new->dest);
 
 	if (snap)
@@ -458,7 +462,7 @@ struct lxc_storage *storage_copy(struct lxc_container *c0, const char *cname,
 
 	data.orig = orig;
 	data.new = new;
-	if (am_unpriv())
+	if (am_guest_unpriv())
 		ret = userns_exec_full(c0->lxc_conf, rsync_rootfs_wrapper,
 				       &data, "rsync_rootfs_wrapper");
 	else
@@ -484,20 +488,21 @@ err:
 struct lxc_storage *storage_create(const char *dest, const char *type,
 				   const char *cname, struct bdev_specs *specs)
 {
+	int ret;
 	struct lxc_storage *bdev;
 	char *best_options[] = {"btrfs", "zfs", "lvm", "dir", "rbd", NULL};
 
 	if (!type)
 		return do_storage_create(dest, "dir", cname, specs);
 
-	if (strcmp(type, "best") == 0) {
+	ret = strcmp(type, "best");
+	if (ret == 0) {
 		int i;
 		/* Try for the best backing store type, according to our
 		 * opinionated preferences.
 		 */
 		for (i = 0; best_options[i]; i++) {
-			bdev = do_storage_create(dest, best_options[i], cname,
-						 specs);
+			bdev = do_storage_create(dest, best_options[i], cname, specs);
 			if (bdev)
 				return bdev;
 		}
@@ -506,12 +511,19 @@ struct lxc_storage *storage_create(const char *dest, const char *type,
 	}
 
 	/* -B lvm,dir */
-	if (strchr(type, ',') != NULL) {
-		char *dup = alloca(strlen(type) + 1), *saveptr = NULL, *token;
-		strcpy(dup, type);
+	if (strchr(type, ',')) {
+		char *dup, *token;
+		char *saveptr = NULL;
+		size_t len;
+
+		len = strlen(type);
+		dup = alloca(len + 1);
+		(void)strlcpy(dup, type, len + 1);
+
 		for (token = strtok_r(dup, ",", &saveptr); token;
 		     token = strtok_r(NULL, ",", &saveptr)) {
-			if ((bdev = do_storage_create(dest, token, cname, specs)))
+			bdev = do_storage_create(dest, token, cname, specs);
+			if (bdev)
 				return bdev;
 		}
 	}
