@@ -32,8 +32,6 @@
 #include "log.h"
 #include "arguments.h"
 
-lxc_log_define(lxc_cgroup_ui, lxc);
-
 static int my_checker(const struct lxc_arguments* args)
 {
 	if (!args->argc) {
@@ -95,26 +93,26 @@ int main(int argc, char *argv[])
 	if (my_args.rcfile) {
 		c->clear_config(c);
 		if (!c->load_config(c, my_args.rcfile)) {
-			ERROR("Failed to load rcfile");
+			fprintf(stderr, "Failed to load rcfile\n");
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
 		}
 		c->configfile = strdup(my_args.rcfile);
 		if (!c->configfile) {
-			ERROR("Out of memory setting new config filename");
+			fprintf(stderr, "Out of memory setting new config filename\n");
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
 		}
 	}
 
 	if (!c->may_control(c)) {
-		ERROR("Insufficent privileges to control %s:%s", my_args.lxcpath[0], my_args.name);
+		fprintf(stderr, "Insufficent privileges to control %s:%s\n", my_args.lxcpath[0], my_args.name);
 		lxc_container_put(c);
 		exit(EXIT_FAILURE);
 	}
 
 	if (!c->is_running(c)) {
-		ERROR("'%s:%s' is not running", my_args.lxcpath[0], my_args.name);
+		fprintf(stderr, "'%s:%s' is not running\n", my_args.lxcpath[0], my_args.name);
 		lxc_container_put(c);
 		exit(EXIT_FAILURE);
 	}
@@ -122,7 +120,7 @@ int main(int argc, char *argv[])
 	if ((my_args.argc) > 1) {
 		value = my_args.argv[1];
 		if (!c->set_cgroup_item(c, state_object, value)) {
-			ERROR("failed to assign '%s' value to '%s' for '%s'",
+			fprintf(stderr, "failed to assign '%s' value to '%s' for '%s'\n",
 				value, state_object, my_args.name);
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
@@ -131,7 +129,7 @@ int main(int argc, char *argv[])
 		char buffer[MAXPATHLEN];
 		int ret = c->get_cgroup_item(c, state_object, buffer, MAXPATHLEN);
 		if (ret < 0) {
-			ERROR("failed to retrieve value of '%s' for '%s:%s'",
+			fprintf(stderr, "failed to retrieve value of '%s' for '%s:%s'\n",
 			      state_object, my_args.lxcpath[0], my_args.name);
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
