@@ -46,8 +46,6 @@
 #include "mainloop.h"
 #include "utils.h"
 
-lxc_log_define(lxc_console_ui, lxc);
-
 static char etoc(const char *expr)
 {
 	/* returns "control code" of given expression */
@@ -98,6 +96,7 @@ int main(int argc, char *argv[])
 {
 	int ret;
 	struct lxc_container *c;
+	struct lxc_log log;
 
 	ret = lxc_arguments_parse(&my_args, argc, argv);
 	if (ret)
@@ -106,8 +105,14 @@ int main(int argc, char *argv[])
 	if (!my_args.log_file)
 		my_args.log_file = "none";
 
-	ret = lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
-			   my_args.progname, my_args.quiet, my_args.lxcpath[0]);
+	log.name = my_args.name;
+	log.file = my_args.log_file;
+	log.level = my_args.log_priority;
+	log.prefix = my_args.progname;
+	log.quiet = my_args.quiet;
+	log.lxcpath = my_args.lxcpath[0];
+
+	ret = lxc_log_init(&log);
 	if (ret)
 		return EXIT_FAILURE;
 	lxc_log_options_no_override();
