@@ -76,7 +76,7 @@ static void test_two_locks(void)
 		if (WIFEXITED(status)) {
 			printf("%d exited normally with exit code %d\n", pid,
 				WEXITSTATUS(status));
-			if (WEXITSTATUS(status) == 0)
+			if (WEXITSTATUS(status) != 0)
 				exit(1);
 		} else
 			printf("%d did not exit normally\n", pid);
@@ -122,7 +122,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 	struct stat sb;
-	char *pathname = RUNTIME_PATH "/lock/lxc/var/lib/lxc/";
+	char *pathname = RUNTIME_PATH "/lxc/lock/var/lib/lxc/";
 	ret = stat(pathname, &sb);
 	if (ret != 0) {
 		fprintf(stderr, "%d: filename %s not created\n", __LINE__,
